BEGIN TRANSACTION;

--创建记录表
CREATE TABLE objects_list(
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	path TEXT NOT NULL UNIQUE,
	hash TEXT NOT NULL,
	type TEXT NOT NULL CHECK(type in ('program','library','kmodule','script','other')),
	exectl TEXT NOT NULL CHECK(exectl in ('original', 'verified', 'kysoft', 'trusted', 'parent', 'script', 'unknown')),
	filepro TEXT NOT NULL CHECK(filepro in ('none', 'readonly')),
	rmmodpro INTEGER NOT NULL DEFAULT 0 CHECK(rmmodpro in (0,1)),
	status INTEGER NOT NULL DEFAULT 0 --执行控制或内核模块保护状态：0 正常，1 已篡改，2不存在
);

CREATE TABLE relabel_status(
	relabel INTEGER NOT NULL DEFAULT 0 --标记初始化标识： 0 全盘初始化，1 按照白名单初始化，2 重新初始化执行控制白名单
);

CREATE TABLE IF NOT EXISTS kmod_list(
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    kmod_name TEXT NOT NULL UNIQUE,
    status INTEGER NOT NULL DEFAULT 0 CHECK(status in (0,1)),
    ctime INTEGER  NOT NULL DEFAULT (strftime('%s','now'))
);

CREATE VIEW IF NOT EXISTS kmod_whlist AS 
SELECT id, kmod_name, status, ctime from kmod_list;

CREATE TRIGGER IF NOT EXISTS kmod_whlist_delete 
INSTEAD OF DELETE ON kmod_whlist 
BEGIN 
	UPDATE kmod_list SET status = 0, ctime = strftime('%s','now') WHERE kmod_name=OLD.kmod_name;
END;

CREATE TRIGGER IF NOT EXISTS kmod_whlist_insert_exists 
INSTEAD OF INSERT ON kmod_whlist WHEN EXISTS(select 1 from kmod_list where kmod_name=NEW.kmod_name) 
BEGIN 
	UPDATE kmod_list SET status = NEW.status, ctime = strftime('%s','now') WHERE kmod_name=NEW.kmod_name;
END;

CREATE TRIGGER IF NOT EXISTS kmod_whlist_insert_not_exists 
INSTEAD OF INSERT ON kmod_whlist WHEN NOT EXISTS(select 1 from kmod_list where kmod_name=NEW.kmod_name) 
BEGIN 
    INSERT INTO kmod_list(kmod_name, status) VALUES(NEW.kmod_name, NEW.status);
END;

INSERT into relabel_status(relabel) values(0);

--执行控制功能白名单视图
CREATE VIEW exectl_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where type in ('program', 'library', 'script') and status != 2;

--文件保护功能视图
CREATE VIEW filepro_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where filepro = 'readonly';

--内核模块保护功能白名单视图
CREATE VIEW kmodpro_whlist AS
SELECT id, path, hash, type, exectl, filepro, rmmodpro, status from objects_list where type in ('kmodule') and status != 2;

COMMIT;

--记录更新触发器，更新记录status字段时，若status状态为0，则删除该记录
CREATE TRIGGER delete_objects AFTER UPDATE OF filepro
ON objects_list
BEGIN
	DELETE FROM objects_list where id = new.id and filepro = 'none' and type = 'other';
END;

--执行控制功能白名单视图删除触发器，采用修改exectl字段值为'unknown'替换, 并修改status值为0
CREATE TRIGGER exectl_whlist_delete
INSTEAD OF DELETE ON exectl_whlist
BEGIN
	UPDATE objects_list SET exectl = 'unknown', status = 0 WHERE path=OLD.path;
END;

--执行控制功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER exectl_whlist_insert_exists
INSTEAD OF INSERT ON exectl_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET exectl = NEW.exectl, hash = NEW.hash, status = 0 WHERE path=NEW.path;
END;

--执行控制功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER exectl_whlist_insert_not_exists
INSTEAD OF INSERT ON exectl_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0);
END;

--文件保护功能视图删除触发器，采用修改filepro字段值为none替换， 并修改status值为0
CREATE TRIGGER filepro_whlist_delete
INSTEAD OF DELETE ON filepro_whlist
BEGIN
	UPDATE objects_list SET filepro = 'none' WHERE path=OLD.path;	
END;

--文件保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER filepro_whlist_insert_exists
INSTEAD OF INSERT ON filepro_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET filepro = NEW.filepro WHERE path=NEW.path;
END;

--文件保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER filepro_whlist_insert_not_exists
INSTEAD OF INSERT ON filepro_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, 'unknown', NEW.filepro, 0, 0);
END;

--内核模块保护功能白名单视图删除触发器，采用修改exectl字段为unknown，rmmodpro字段值为0替换，并修改status值为0
CREATE TRIGGER kmodpro_whlist_delete
INSTEAD OF DELETE ON kmodpro_whlist
BEGIN
	UPDATE objects_list SET exectl = 'unknown', status = 0 WHERE path=OLD.path;
END;

--内核模块保护功能白名单视图插入触发器，当记录已存在时，修改已存在记录exectl为新exectl状态， 并修改status值为1
CREATE TRIGGER kmodpro_whlist_insert_exists
INSTEAD OF INSERT ON kmodpro_whlist WHEN EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	UPDATE objects_list SET exectl = NEW.exectl, hash = NEW.hash, status = 0 WHERE path=NEW.path;
END;

--内核模块保护功能白名单视图插入触发器，当记录不存在时，在表中插入记录
CREATE TRIGGER kmodpro_whlist_insert_not_exists
INSTEAD OF INSERT ON kmodpro_whlist WHEN NOT EXISTS(select 1 from objects_list where path=NEW.path)
BEGIN
	INSERT INTO objects_list(path, hash, type, exectl, filepro, rmmodpro, status) values(NEW.path, NEW.hash, NEW.type, NEW.exectl, 'none', 0, 0);
END;

